@@ -217,7 +217,6 @@ def test_gld_dtm(idxrow, idxcol, row0, col0, steprow, stepcol, nbrow, nbcol):
     Test loc direct grid on dtm function
     """
     dtmbsq, gri = prepare_loc()
-
     gri_gld = gri.direct_loc_grid_dtm(row0, col0, steprow, stepcol, nbrow, nbcol, dtmbsq)
 
     lonlatalt = gri_gld[:, idxrow, idxcol]
@@ -239,12 +238,9 @@ def test_loc_dir_instersect_cube_dtm(col, row, valid_lon, valid_lat, valid_alt):
     """
     dtmbsq, gri = prepare_loc()
 
-    los = np.zeros((3, gri.nbalt))
-    vislonlat = gri.interpolate_grid_in_plani(row, col)
-    los[0, :] = vislonlat[0]
-    los[1, :] = vislonlat[1]
-    los[2, :] = gri.alts_down
-    los = los.T
+    los = np.zeros((gri.nbalt, 3))
+    los[:, 0:2] = gri.interpolate_grid_in_plani(row, col)
+    los[:, 2] = gri.alts_down
     (__, position, __, __) = dtmbsq.intersect_dtm_cube(los)
     position = dtmbsq.index_to_ter(position)
     assert position[0] == pytest.approx(valid_lon, abs=1e-12)
@@ -260,9 +256,9 @@ def test_loc_dir_interp_visee_unitaire_gld(row, col, valid_lon, valid_lat):
     Test los interpolation
     """
     ___, gri = prepare_loc()
-    los = gri.interpolate_grid_in_plani(row, col)
-    assert los[0][1] == pytest.approx(valid_lon, abs=1e-12)
-    assert los[1][1] == pytest.approx(valid_lat, abs=1e-12)
+    los = np.squeeze(gri.interpolate_grid_in_plani(row, col))
+    assert los[1, 0] == pytest.approx(valid_lon, abs=1e-12)
+    assert los[1, 1] == pytest.approx(valid_lat, abs=1e-12)
 
 
 @pytest.mark.parametrize("col,row,h", [(50.5, 100.5, 100.0)])
@@ -534,8 +530,8 @@ def test_sensor_loc_inv_vs_loc_rpc(lon, lat, alt):
     diff_row = row_rpc - row_rpc_optim
     diff_col = col_rpc - col_rpc_optim
 
-    assert diff_row == pytest.approx(0.0, abs=4e-12)
-    assert diff_col == 0.0
+    assert diff_row == pytest.approx(0.0, abs=8e-12)
+    assert diff_col == pytest.approx(0.0, abs=8e-12)
 
 
 @pytest.mark.parametrize(
@@ -591,12 +587,9 @@ def test_loc_intersection(row, col, valid_lon, valid_lat, valid_alt):
     """
     dtmbsq, gri = prepare_loc()
 
-    los = np.zeros((3, gri.nbalt))
-    vislonlat = gri.interpolate_grid_in_plani(row, col)
-    los[0, :] = vislonlat[0]
-    los[1, :] = vislonlat[1]
-    los[2, :] = gri.alts_down
-    los = los.T
+    los = np.zeros((gri.nbalt, 3))
+    los[:, 0:2] = gri.interpolate_grid_in_plani(row, col)
+    los[:, 2] = gri.alts_down
     (__, point_b, alti, los_index) = dtmbsq.intersect_dtm_cube(los)
     (__, point_dtm) = dtmbsq.intersection(los_index, point_b, alti)
     assert point_dtm[0] == pytest.approx(valid_lon, abs=1e-12)
@@ -731,7 +724,7 @@ def test_sensor_coloc_using_geotransform(col, row, h):
         geom_model_left_optim, geom_model_right_optim, row, col, h, image_left, image_right, using_geotransform=True
     )
     np.testing.assert_allclose(row_coloc, row_coloc_optim, 0, 8e-10)
-    np.testing.assert_allclose(col_coloc, col_coloc_optim, 0, 2e-11)
+    np.testing.assert_allclose(col_coloc, col_coloc_optim, 0, 3e-11)
 
     origin_left = [5000.0, 5000.0]
     pix_size_left = [0.5, 0.5]
@@ -774,8 +767,8 @@ def test_sensor_loc_utm(col, row):
     inv_row, inv_col, __ = loc_utm.inverse(coord_utm[:, 0], coord_utm[:, 1])
     inv_row_optim, inv_col_optim, __ = loc_utm_optim.inverse(coord_utm_optim[:, 0], coord_utm_optim[:, 1])
 
-    np.testing.assert_allclose(inv_row, inv_row_optim, 0, 8e-12)
-    np.testing.assert_allclose(inv_col, inv_col_optim, 0, 8e-12)
+    np.testing.assert_allclose(inv_row, inv_row_optim, 0, 2e-11)
+    np.testing.assert_allclose(inv_col, inv_col_optim, 0, 2e-11)
 
     assert row == pytest.approx(inv_row[0], abs=1e-8)
     assert col == pytest.approx(inv_col[0], abs=1e-8)
